package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.highlevel.controller.JFedHighLevelException;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.util.AggregateManagerWrapperFactory;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedException;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.AggregateManagerWrapper;
import be.iminds.ilabt.jfed.util.common.GeniUrn;

import javax.annotation.Nonnull;
import java.util.Collections;
import java.util.List;

/**
 * User: twalcari
 * Date: 6/17/2015
 * Time: 16:38
 */
public class ReloadOSTask extends Task {
    @Nonnull private final Slice slice;
    @Nonnull private final Server auth;
    @Nonnull private final GeniUrn reloadOSTarget;

    @Nonnull private final AggregateManagerWrapperFactory aggregateManagerWrapperFactory;
    @Nonnull private final HighLevelTaskFactory highLevelTaskFactory;

    public ReloadOSTask(@Nonnull Slice slice,
                        @Nonnull Server auth,
                        @Nonnull GeniUrn reloadOSTarget,
                        @Nonnull AggregateManagerWrapperFactory aggregateManagerWrapperFactory,
                        @Nonnull HighLevelTaskFactory highLevelTaskFactory) {
        super("Reload OS on " + reloadOSTarget);
        this.slice = slice;
        this.auth = auth;
        this.reloadOSTarget = reloadOSTarget;
        this.aggregateManagerWrapperFactory = aggregateManagerWrapperFactory;
        this.highLevelTaskFactory = highLevelTaskFactory;
    }

    @Override
    public void doTask(@Nonnull TaskExecution taskExecution) throws JFedException, InterruptedException {
        AggregateManagerWrapper amWrapper =
                aggregateManagerWrapperFactory.getAggregateManagerWrapper(taskExecution, auth);
        if (!amWrapper.isReloadOSSupported())
            throw new JFedHighLevelException("Authority " + auth.getDefaultComponentManagerUrn() + " does not support reloadOS.");
        amWrapper.reloadOS(reloadOSTarget, slice.getCredentials());
    }

    @Nonnull
    @Override
    public List<Task> initDependsOn() {
        return Collections.singletonList(highLevelTaskFactory.getSliceCredential(slice));
    }
}
